==> core-0.4.5.tgz: package/lib/cache/Memcache.js <==
var Class = require('../class/Class.js').Class;
var assertions = require('../common/Assertions.js');
var types = require('../common/Types.js');
var net = require('net');
var ConnectionPool = require('../net/ConnectionPool.js').ConnectionPool;



var Memcache = function(){
	
	this._lifetime = 10;
    this._connectionPool = null;

	this.initialize = function(pool){
		this._connectionPool = pool;
	};
    
	this.get = function(key, callback){
        //sprawdzam w this._memory czy jest key
        //jesli nie to dalej
        //jesli jest to wisimy czyli callback 
        assertions.isString(key, Memcache.Exception.WRONG_TYPE);
        
        
        var that = this;
        
        this._connectionPool.getSocket(function(socket){
            
            
            var dataLength = 0, buffers = [];
            
            socket.on('data', function(e){

                // jezeli E(nd) lub C(ommand bad format)
                if(e[0]==69 || e[0]==67){
                    that._connectionPool.givebackSocket(socket);
                    callback(null);
                    return;
                }
                
                dataLength += e.length;
                buffers.push(e);
                
                if(e.slice(e.length - 5).toString('ascii')==='END\r\n'){
                    
                    that._connectionPool.givebackSocket(socket);
                    
                    var _buffer = new Buffer(dataLength);
                    var bufferPosition = 0;
                    for(var i = 0, l = buffers.length; i < l; i++){
         			    buffers[i].copy(_buffer, bufferPosition);
         			    bufferPosition += buffers[i].length;
         		     }
                    
                    var dataStart = 0;
                    for(var i=0;i<_buffer.length;i++){
                        if(_buffer[i]===13 && _buffer[i+1]===10){
                            dataStart = i+2;
                            break;
                        }
                    }
                    callback(_buffer.slice(dataStart, _buffer.length-7));
                }                               
                
            });
            
            var resultBuff = new Buffer('get ' + key +'\r\n', 'binary');
            socket.write(resultBuff);
        });
        
	};

	this.set = function(key, value, callback, lifeTime, waitForReply){
	     //TODO dac zeby bylo reply i wtedy oddawac socket
		  assertions.isString(key, Memcache.Exception.WRONG_TYPE);
        
        lifeTime = lifeTime || this._lifetime;
        waitForReply = waitForReply || false;
       
        var that = this;
        
        this._connectionPool.getSocket(function(socket){
            
            if(!Buffer.isBuffer(value)){
                value = new Buffer(value, 'utf8');
            }
            
            var cmdBuffStart = new Buffer('set ' + key + ' 0 ' + lifeTime + ' ' + value.length + (!waitForReply ? ' noreply' : '') + '\r\n', 'binary');
            var cmdBuffEnd = new Buffer('\r\n', 'binary');
            var resultBuff = new Buffer(cmdBuffStart.length + value.length + cmdBuffEnd.length);
            cmdBuffStart.copy(resultBuff);
            value.copy(resultBuff, cmdBuffStart.length);
            cmdBuffEnd.copy(resultBuff, (cmdBuffStart.length + value.length));

            var dataLength = 0, buffers = [];
            
            if (waitForReply) {
                socket.on('data', function(e){
                    // jezeli S(tored))
                    if(e[0]== 83 && e[1] == 84){
                        that._connectionPool.givebackSocket(socket);
                        callback(true);
                        return;
                    }

                    dataLength += e.length;
                    buffers.push(e);

                    if(e.slice(e.length - 2).toString('ascii')==='\r\n'){

                        that._connectionPool.givebackSocket(socket);

                        var _buffer = new Buffer(dataLength), bufferPosition = 0;
                        for(var i = 0, l = buffers.length; i < l; i++){
                            buffers[i].copy(_buffer, bufferPosition);
                            bufferPosition += buffers[i].length;
                         }

                        callback(false, _buffer.slice(0, _buffer.length-2));
                    }                               
                });
            }
            
            socket.write(resultBuff, function () {
                if (!waitForReply) {
                    that._connectionPool.givebackSocket(socket);
                }
            });
        });
	};
    
    this['delete'] = function(key, callback){
        var that = this;
        
        //TODO dac zeby bylo reply i wtedy oddawac socket
		assertions.isString(key, Memcache.Exception.WRONG_TYPE);
        
        this._connectionPool.getSocket(function(socket){
            var resultBuff = new Buffer('delete ' + key + ' noreply\r\n', 'binary');
            socket.write(resultBuff, function(){
                that._connectionPool.givebackSocket(socket);
            });
        });
        
    };

	this.setLifetime = function(lifetime){
		assertions.isNumber(lifetime, Memcache.Exception.WRONG_LIFETIME_TYPE);
		this._lifetime = lifetime;
		return this;
	};

	this.getLifetime = function(){
		return this._lifetime;
	};
};
Memcache = new Class(new Memcache());
Memcache.Exception = {};
Memcache.Exception.WRONG_KEY_TYPE = "Key has to be a string";
Memcache.Exception.WRONG_VALUE_TYPE = "Value has to be a string, number or object";
Memcache.Exception.WRONG_LIFETIME_TYPE = "Lifetime has to be a number";
exports.Memcache = Memcache;

==> core-0.4.6.tgz: package/lib/cache/Memcache.js <==
var Class = require('../class/Class.js').Class;
var assertions = require('../common/Assertions.js');
var Types = require('../common/Types.js');
var net = require('net');
var ConnectionPool = require('../net/ConnectionPool.js').ConnectionPool;



var Memcache = function(){
	
	this._lifetime = 10;
    this._connectionPool = null;

	this.initialize = function(pool){
		this._connectionPool = pool;
	};
    
	this.get = function(key, callback){
        //sprawdzam w this._memory czy jest key
        //jesli nie to dalej
        //jesli jest to wisimy czyli callback 
        assertions.isString(key, Memcache.Exception.WRONG_TYPE);
        
        
        var that = this;
        
        this._connectionPool.getSocket(function(socket){
            
            
            var dataLength = 0, buffers = [];
            
            socket.on('data', function(e){

                // jezeli E(nd) lub C(ommand bad format)
                if(e[0]==69 || e[0]==67){
                    that._connectionPool.givebackSocket(socket);
                    callback(null);
                    return;
                }
                
                dataLength += e.length;
                buffers.push(e);
                
                if(e.slice(e.length - 5).toString('ascii')==='END\r\n'){
                    
                    that._connectionPool.givebackSocket(socket);
                    
                    var _buffer = new Buffer(dataLength);
                    var bufferPosition = 0;
                    for(var i = 0, l = buffers.length; i < l; i++){
         			    buffers[i].copy(_buffer, bufferPosition);
         			    bufferPosition += buffers[i].length;
         		     }
                    
                    var dataStart = 0;
                    for(var i=0;i<_buffer.length;i++){
                        if(_buffer[i]===13 && _buffer[i+1]===10){
                            dataStart = i+2;
                            break;
                        }
                    }
                    callback(_buffer.slice(dataStart, _buffer.length-7));
                }                               
                
            });
            
            var resultBuff = new Buffer('get ' + key +'\r\n', 'binary');
            socket.write(resultBuff);
        });
        
	};

	this.set = function(key, value, callback, lifeTime, waitForReply){
	     //TODO dac zeby bylo reply i wtedy oddawac socket
		  assertions.isString(key, Memcache.Exception.WRONG_TYPE);
        
        lifeTime = (Types.isNumber(lifeTime)) ? lifeTime : this._lifetime;
        waitForReply = waitForReply || false;
       
        var that = this;
        
        this._connectionPool.getSocket(function(socket){
            
            if(!Buffer.isBuffer(value)){
                value = new Buffer(value, 'utf8');
            }
            
            var cmdBuffStart = new Buffer('set ' + key + ' 0 ' + lifeTime + ' ' + value.length + (!waitForReply ? ' noreply' : '') + '\r\n', 'binary');
            var cmdBuffEnd = new Buffer('\r\n', 'binary');
            var resultBuff = new Buffer(cmdBuffStart.length + value.length + cmdBuffEnd.length);
            cmdBuffStart.copy(resultBuff);
            value.copy(resultBuff, cmdBuffStart.length);
            cmdBuffEnd.copy(resultBuff, (cmdBuffStart.length + value.length));

            var dataLength = 0, buffers = [];
            
            if (waitForReply) {
                socket.on('data', function(e){
                    // jezeli S(tored))
                    if(e[0]== 83 && e[1] == 84){
                        that._connectionPool.givebackSocket(socket);
                        callback(true);
                        return;
                    }

                    dataLength += e.length;
                    buffers.push(e);

                    if(e.slice(e.length - 2).toString('ascii')==='\r\n'){

                        that._connectionPool.givebackSocket(socket);

                        var _buffer = new Buffer(dataLength), bufferPosition = 0;
                        for(var i = 0, l = buffers.length; i < l; i++){
                            buffers[i].copy(_buffer, bufferPosition);
                            bufferPosition += buffers[i].length;
                         }

                        callback(false, _buffer.slice(0, _buffer.length-2));
                    }                               
                });
            }
            
            socket.write(resultBuff, function () {
                if (!waitForReply) {
                    that._connectionPool.givebackSocket(socket);
                }
            });
        });
	};
    
    this['delete'] = function(key, callback){
        var that = this;
        
        //TODO dac zeby bylo reply i wtedy oddawac socket
		assertions.isString(key, Memcache.Exception.WRONG_TYPE);
        
        this._connectionPool.getSocket(function(socket){
            var resultBuff = new Buffer('delete ' + key + ' noreply\r\n', 'binary');
            socket.write(resultBuff, function(){
                that._connectionPool.givebackSocket(socket);
            });
        });
        
    };

	this.setLifetime = function(lifetime){
		assertions.isNumber(lifetime, Memcache.Exception.WRONG_LIFETIME_TYPE);
		this._lifetime = lifetime;
		return this;
	};

	this.getLifetime = function(){
		return this._lifetime;
	};
};
Memcache = new Class(new Memcache());
Memcache.Exception = {};
Memcache.Exception.WRONG_KEY_TYPE = "Key has to be a string";
Memcache.Exception.WRONG_VALUE_TYPE = "Value has to be a string, number or object";
Memcache.Exception.WRONG_LIFETIME_TYPE = "Lifetime has to be a number";
exports.Memcache = Memcache;
